#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

require 'zk'
require 'getopt/std'
require 'tempfile'
require 'combine_pdf'

class IncidentGroupReportsController < ApplicationController
  include Exportable

  ## ======================================================================
  # => INCLUDES
  ## ======================================================================

  include AlarmsHelper
  include Redborder::Filters::Storage::Session


  helper GroverHelper::GroverAssetsHelper

  ## ======================================================================
  # => Controller properties
  ## ======================================================================

  respond_to :html
  respond_to :js, :json,
             only: [:create, :update, :destroy, :update]

  layout false, only: [:new_user, :new_sensor, :send_now]

  # layout false, only: [:import, :send_now, :new_user, :new_sensor, :generate,
  #                      :remove_page, :remove_page_modal, :sensor_childrens]

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :set_report,
                only: [:show, :edit, :destroy, :update, :send_now]


  def new 
    @incident_group_report = IncidentGroupReport.new
  end

  def create
    # Get all the params
    to_save = create_report_params

    # Default values
    to_save.merge!(
      number_pages: 1,
    )

    # Save the collaborators and the sensors
    collaborators = to_save.delete(:collaborators)
    sensors_collaborators = to_save.delete(:sensors_collaborators)

    # Create the Report
    @incident_group_report = IncidentGroupReport.new(to_save)

    # Associate the user
    @incident_group_report.owner = current_user

    if @incident_group_report.save
      save_collaborators @incident_group_report, collaborators
      save_sensors_collaborators @incident_group_report, sensors_collaborators
      flash.now[:notice] = I18n.t('incident_group_report.create.success')
      # Force changing the tab to Incident Reports
      redirect_to reports_path
    end
  end

  def edit
  end

  def update
     # Get all the params
    to_save = create_report_params

    # Default values
    to_save.merge!(
      number_pages: 1,
    )

    # Save the collaborators and the sensors
    collaborators = to_save.delete(:collaborators)
    sensors_collaborators = to_save.delete(:sensors_collaborators)

    if to_save.key? "collaborators"
      to_save["collaborators"].reject! { |c| c["user_id"] == user_id.to_s }
    end

    if @incident_group_report.update to_save
      save_collaborators @incident_group_report, collaborators
      save_sensors_collaborators @incident_group_report, sensors_collaborators
      flash.now[:notice] = I18n.t('incident_group_report.create.success')
      # Force changing the tab to Incident Reports
      redirect_to reports_path
    end
  end

  def destroy
    if @incident_group_report.owner == current_user
      @incident_group_report.destroy
    else
      @incident_group_report.report_users.find_by(user_id: current_user.id).try(:destroy)
    end

    respond_with @incident_group_report, location: reports_url
  end

  def show
  respond_to do |format|
    format.pdf do
      generator = IncidentReportGenerator.new() 
      time = Time.now
      time_past = case @incident_group_report.schedule_type
      when "daily"
        Time.now - 1.days
      when "weekly"
        Time.now - 7.days
      when "monthly"
        Time.now - 1.months
      end
      start_date = time_past.strftime("%d-%m-%y")
      end_date = time.strftime("%d-%m-%y")

      report_name = generator.get_report_name start_date, end_date, nil, @incident_group_report.report_name
      pdf = generator.generate_group_pdf @incident_group_report, current_user, report_name

      Tempfile.open(['temp', '.pdf']) do |file|
        file.binmode
        file.write(pdf)
        file.rewind
        @incident_group_report.create_incident_historical(current_user, file, report_name)
      end
      
      # send_data combined_pdf, filename: "#{report_name}" + '.pdf',
      send_data pdf, filename: "#{report_name}" + '.pdf',
      type: 'application/pdf',
      disposition: 'inline'

      end
    end
  end
  


  def create_report_params
    schedule_params
    params.require(:incident_group_report)
      .permit(:name, :description,
              :schedule_type, :email_list, :schedule_days, :schedule_hour,
              :condition, :report_name,
              collaborators: [:user_id, :can_edit],
              sensors_collaborators: [:sensor_id, :can_edit])
  end

  def schedule_params
    hour = params[:incident_group_report_hour]
    minute = params[:incident_group_report_minute]
    params[:incident_group_report][:schedule_hour] = "#{hour}:#{minute}"
  end

  def save_collaborators(report, collaborators)
    # Remove all relations, except those with the owner and the current user
    report.report_users.each do |x|
      x.delete unless [x.incident_group_report.user_id, current_user.id].include? x.user_id
    end

    return if collaborators.nil?

    # Dont save relations with the owner or the current user
    collaborators.each do |user|
      next if user['user_id'].to_i == report.user_id

      report.report_users.create(
        user_id: user['user_id'],
        can_edit: (user['can_edit'] == 'on')
      )
    end
  end

  def save_sensors_collaborators(report, sensors_collaborators)
    # Remove all relations
    report.report_sensors.destroy_all
    return if sensors_collaborators.nil?

    # Save all relations
    sensors_collaborators.each do |_index, sensor|
      next unless [:all, :admin, :none].include? sensor['can_edit'].to_sym

      report.report_sensors.create(
        sensor_id: sensor['sensor_id'],
        can_edit: sensor['can_edit']
      )
    end
  end

  def new_sensor
    @sensors = if current_user.super_admin?
                 current_user.sensor.children.where(type: Sensor::DOMAIN)
                 .reject { |s| (s.unassigned? && s.children.count.zero?) || s.is_root? }
               else
                 [current_user.sensor]
               end
  end

  def send_now
    Delayed::Job.enqueue(Redborder::Jobs::SendReportJob
      .new(@incident_group_report.id, current_user, 'incident', true), priority: 10, run_at: Time.now)
  end

  private

  def set_report
    @incident_group_report = IncidentGroupReport.find(params[:id])
  end

end